<?php

namespace Drupal\Tests\field_event_dispatcher\Kernel;

use Drupal\Core\Entity\Display\EntityFormDisplayInterface;
use Drupal\Core\Field\WidgetInterface;
use Drupal\Core\Form\FormState;
use Drupal\field_event_dispatcher\Event\Field\WidgetFormAlterEvent;
use Drupal\field_event_dispatcher\Event\Field\WidgetMultivalueFormAlterEvent;
use Drupal\hook_event_dispatcher\HookEventDispatcherInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\hook_event_dispatcher\Kernel\ListenerTrait;

/**
 * Class WidgetFormAlterEventTest.
 *
 * @group hook_event_dispatcher
 * @group field_event_dispatcher
 *
 * @see \field_event_dispatcher_field_widget_form_alter()
 * @see \field_event_dispatcher_field_widget_multivalue_form_alter()
 */
class WidgetFormAlterEventTest extends KernelTestBase {

  use ListenerTrait;

  protected const ENTITY_TYPE_ID = 'entity_test_base_field_display';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'text',
    'user',
    'entity_test',
    'hook_event_dispatcher',
    'field_event_dispatcher',
  ];

  /**
   * Test WidgetFormAlterEvent.
   *
   * @throws \Exception
   */
  public function testWidgetFormAlterEvent(): void {
    $this->listen([
      HookEventDispatcherInterface::WIDGET_FORM_ALTER,
      'hook_event_dispatcher.widget_text.alter',
    ], 'onWidgetFormAlter', $this->exactly(2));
    $this->listen(HookEventDispatcherInterface::WIDGET_MULTIVALUE_FORM_ALTER, 'onWidgetMultiValueFormAlter');

    $entityTypeManager = $this->container->get('entity_type.manager');

    $entity = $entityTypeManager->getStorage(self::ENTITY_TYPE_ID)->create();

    $entityFormDisplay = $entityTypeManager->getStorage('entity_form_display')->create([
      'targetEntityType' => self::ENTITY_TYPE_ID,
      'bundle' => self::ENTITY_TYPE_ID,
    ]);
    $this->assertInstanceOf(EntityFormDisplayInterface::class, $entityFormDisplay);
    $entityFormDisplay->setComponent('test_display_configurable', []);

    $form = [];
    $formState = new FormState();
    $formState->set('test', TRUE);
    $entityFormDisplay->buildForm($entity, $form, $formState);

    $this->assertArrayHasKey('test_display_configurable', $form);

    $widget = $form['test_display_configurable']['widget'][0];
    $this->assertArrayHasKey('other', $widget);
    $this->assertEquals('key', $widget['other']);
    $this->assertArrayHasKey('other_element', $widget);
    $this->assertEquals('key', $widget['other']);
  }

  /**
   * Callback for WidgetFormAlterEvent.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\WidgetFormAlterEvent $event
   *   The event.
   */
  public function onWidgetFormAlter(WidgetFormAlterEvent $event) {
    $formState = $event->getFormState();
    $this->assertTrue($formState->has('test'));
    $this->assertTrue($formState->get('test'));

    $context = $event->getContext();
    $widget = $context['widget'];
    $this->assertInstanceOf(WidgetInterface::class, $widget);
    $this->assertEquals('text_textfield', $widget->getPluginId());

    $element = &$event->getElement();
    $element['other'] = 'key';
  }

  /**
   * Callback for WidgetMultivalueFormAlterEvent.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\WidgetMultivalueFormAlterEvent $event
   *   The event.
   */
  public function onWidgetMultiValueFormAlter(WidgetMultivalueFormAlterEvent $event) {
    $formState = $event->getFormState();
    $this->assertTrue($formState->has('test'));
    $this->assertTrue($formState->get('test'));

    $context = $event->getContext();
    $widget = $context['widget'];
    $this->assertInstanceOf(WidgetInterface::class, $widget);
    $this->assertEquals('text_textfield', $widget->getPluginId());

    $elements = &$event->getElements();
    $elements[0]['other_element'] = 'key';
  }

}
